#pragma once

#include <lektor/defines.h>
#include <lektor/net.h>
#include <lektor/window.h>

#include <stdbool.h>

/**
 * The list of commands implemented by lektor.
 *
 * Refer to the following link for the description of each command:
 * <https://www.musicpd.org/doc/html/protocol.html>
 */

/* Querying lektor's status */
bool command_currentsong(struct lkt_state *srv, size_t c);
bool command_status(struct lkt_state *srv, size_t c);

/* Controlling playback */
bool command_next(sqlite3 *db, struct lkt_win *win, enum mpd_idle_flag *watch_mask_ptr);
bool command_pause(sqlite3 *db, struct lkt_win *win, enum mpd_idle_flag *watch_mask_ptr);
bool command_previous(sqlite3 *db, struct lkt_win *win, enum mpd_idle_flag *watch_mask_ptr);
bool command_play(sqlite3 *db, struct lkt_win *win, char *args[LKT_MESSAGE_ARGS_MAX],
                  enum mpd_idle_flag *watch_mask_ptr);
bool command_stop(sqlite3 *db, struct lkt_win *win, enum mpd_idle_flag *watch_mask_ptr);

bool command_set_pos(sqlite3 *db, struct lkt_win *win, enum mpd_idle_flag *watch_mask_ptr, int index);

/* The queue */
bool command_add(sqlite3 *db, struct lkt_win *win, char *args[LKT_MESSAGE_ARGS_MAX],
                 enum mpd_idle_flag *watch_mask_ptr);
bool command_addid(sqlite3 *db, struct lkt_win *win, char *args[LKT_MESSAGE_ARGS_MAX],
                   enum mpd_idle_flag *watch_mask_ptr);
bool command_del(sqlite3 *db, struct lkt_win *win, char *pos_range, enum mpd_idle_flag *watch_mask_ptr);
bool command_delid(sqlite3 *db, struct lkt_win *win, char *id, enum mpd_idle_flag *watch_mask_ptr);
bool command_clear(sqlite3 *db, enum mpd_idle_flag *watch_mask_ptr);
bool command_crop(sqlite3 *db, enum mpd_idle_flag *watch_mask_ptr);
bool command_move(sqlite3 *db, char *args[LKT_MESSAGE_ARGS_MAX], enum mpd_idle_flag *watch_mask_ptr);
bool command_shuffle(sqlite3 *db, enum mpd_idle_flag *watch_mask_ptr);
bool command_queue_list(struct lkt_state *srv, size_t c, char *args[LKT_MESSAGE_ARGS_MAX]);
bool command_playid(sqlite3 *db, struct lkt_win *win, char *args[LKT_MESSAGE_ARGS_MAX],
                    enum mpd_idle_flag *watch_mask_ptr);
bool command_queue_find(struct lkt_state *srv, size_t c, char *cmd_args[LKT_MESSAGE_ARGS_MAX],
                        long continuation);

/* The playlists */
bool command_plt_create(sqlite3 *db, char *args[LKT_MESSAGE_ARGS_MAX], enum mpd_idle_flag *watch_mask_ptr);
bool command_plt_remove(sqlite3 *db, char *args[LKT_MESSAGE_ARGS_MAX], enum mpd_idle_flag *watch_mask_ptr);
bool command_plt_clear(sqlite3 *db, char *args[LKT_MESSAGE_ARGS_MAX], enum mpd_idle_flag *watch_mask_ptr);
bool command_plt_rename(sqlite3 *db, char *args[LKT_MESSAGE_ARGS_MAX], enum mpd_idle_flag *watch_mask_ptr);
bool command_plt_export(sqlite3 *db, char *args[LKT_MESSAGE_ARGS_MAX], enum mpd_idle_flag *watch_mask_ptr);
bool command_plt_import(sqlite3 *db, char *args[LKT_MESSAGE_ARGS_MAX], enum mpd_idle_flag *watch_mask_ptr);
bool command_plt_add(sqlite3 *db, char *args[LKT_MESSAGE_ARGS_MAX], enum mpd_idle_flag *watch_mask_ptr);

/* The help */
bool command_help(struct lkt_state *srv, size_t c);

/* Update the database */
bool command_update(struct lkt_state *srv, enum mpd_idle_flag *watch_mask_ptr);

/* Respond to the idle command */
bool command_idle(struct lkt_state *srv, size_t c, struct lkt_command *cmd);

/* Cancel the call to the `idle` command. */
bool command_noidle(struct lkt_state *srv, size_t c);

/* Find commands */
enum lkt_find_action {
    LKT_FND_ACT_NONE = 0,
    LKT_FND_ACT_RESPOND = 1,
    LKT_FND_ACT_PRINT = 2,
    LKT_FND_ACT_ENQUEUE = 3,
};

/* Find and send karas in the db that match the search expression */
bool command_find(struct lkt_state *srv, size_t c, char *cmd_args[LKT_MESSAGE_ARGS_MAX],
                  long continuation, enum lkt_find_action action);

/* Set options for the lektor such as `random`, `single`, `repeat`, etc */
enum lkt_playback_option {
    lkt_playback_option_none,
    lkt_playback_option_random,
    lkt_playback_option_single,
    lkt_playback_option_repeat,
    lkt_playback_option_consume,
    lkt_playback_option_volume,
};

bool command_set_playback_option(struct lkt_state *srv, size_t c, enum lkt_playback_option opt,
                                 char *args[LKT_MESSAGE_ARGS_MAX]);

/* Authentificate users */
bool command_password(struct lkt_state *srv, size_t c, char *argv[LKT_MESSAGE_ARGS_MAX]);
bool command_user_add(struct lkt_state *srv, size_t c, sqlite3 *db, char *argv[LKT_MESSAGE_ARGS_MAX]);

/* Program management control */
bool command_restart(struct lkt_state *srv, size_t c);
bool command_kill(struct lkt_state *srv, size_t c);
bool command_rescan(struct lkt_state *srv, size_t c, char *argv[LKT_MESSAGE_ARGS_MAX]);

/* Sticker management */
bool command_sticker_get(struct lkt_state *srv, size_t c, char *argv[LKT_MESSAGE_ARGS_MAX]);
bool command_sticker_set(struct lkt_state *srv, size_t c, char *argv[LKT_MESSAGE_ARGS_MAX]);
bool command_sticker_delete(struct lkt_state *srv, size_t c, char *argv[LKT_MESSAGE_ARGS_MAX]);
bool command_sticker_list(struct lkt_state *srv, size_t c, char *argv[LKT_MESSAGE_ARGS_MAX]);
